@@ -5,6 +5,11 @@
 
 #include "kernel.h"
 
+enum
+{
+	MAX_TIMELINE_MARKERS=64
+};
+
 struct CDemoHeader
 {
 	unsigned char m_aMarker[7];
@@ -18,6 +23,12 @@ struct CDemoHeader
 	char m_aTimestamp[20];
 };
 
+struct CTimelineMarkers
+{
+	char m_aNumTimelineMarkers[4];
+	char m_aTimelineMarkers[MAX_TIMELINE_MARKERS][4];
+};
+
 class IDemoPlayer : public IInterface
 {
 	MACRO_INTERFACE("demoplayer", 0)
@@ -31,6 +42,9 @@ public:
 		int m_FirstTick;
 		int m_CurrentTick;
 		int m_LastTick;
+
+		int m_NumTimelineMarkers;
+		int m_aTimelineMarkers[MAX_TIMELINE_MARKERS];
 	};
 
 	enum
@@ -42,7 +56,7 @@ public:
 
 	~IDemoPlayer() {}
 	virtual void SetSpeed(float Speed) = 0;
-	virtual int SetPos(float Precent) = 0;
+	virtual int SetPos(float Percent) = 0;
 	virtual void Pause() = 0;
 	virtual void Unpause() = 0;
 	virtual const CInfo *BaseInfo() const = 0;
